@@ -21,8 +21,6 @@
 
 from __future__ import absolute_import
 
-import sys
-import logging
 from argparse import ArgumentParser
 
 from gevent import monkey; monkey.patch_all()
@@ -31,28 +29,27 @@ from gevent import sleep
 from .core import VERSION
 from .state import SlimtaState
 
-logging.basicConfig(level=logging.DEBUG, stream=sys.stdout)
-
 
 def parse_args():
     argparser = ArgumentParser(description='Configurable MTA based on the python-slimta libraries.')
     argparser.add_argument('--version', action='version', version='%(prog)s '+VERSION)
     argparser.add_argument('-c', '--config', metavar='FILE', default=None,
-                    help='Specifies a configuration file to read. If not given, the default locations ($HOME/.slimta.conf, f/etc/slimta.conf) are checked.')
-    argparser.add_argument('-a', '--attached', action='store_true',
-                    help='Prevent process from daemonizing, overriding configs.')
+                    help='Specifies a configuration file to read. If not given, the default locations ($HOME/.slimta/slimta.conf, /etc/slimta/slimta.conf) are checked.')
+    argparser.add_argument('-a', '--no-daemon', dest='attached', action='store_true',
+                    help='Override configs and force the process to remain attached to the terminal.')
+    argparser.add_argument('-p', '--pid-file', metavar='FILE', default=None,
+                    help='Store process ID in FILE during execution.')
+
     return argparser, argparser.parse_args()
 
 
-def main():
-    argparser, args = parse_args()
-
-    state = SlimtaState('slimta', args.attached)
-    if not state.load_config(args.config):
-        argparser.error('No configuration files found!')
+def slimta():
+    state = SlimtaState('slimta')
+    state.load_config(*parse_args())
 
     state.start_edges()
 
+    state.setup_logging()
     state.redirect_streams()
     state.daemonize()
     sleep(0.1)
@@ -61,4 +58,19 @@ def main():
     state.loop()
 
 
+def worker():
+    state = SlimtaState('worker')
+    state.load_config(*parse_args())
+
+    state.start_celery_queues()
+
+    state.setup_logging()
+    state.redirect_streams()
+    state.daemonize()
+    sleep(0.1)
+    state.drop_privileges()
+
+    state.worker_loop()
+
+
 # vim:et:fdm=marker:sts=4:sw=4:ts=4
