@@ -79,7 +79,8 @@ class ConfigValidation(object):
                    'user': (basestring, False),
                    'group': (basestring, False),
                    'stdout': (basestring, False),
-                   'stderr': (basestring, False)}
+                   'stderr': (basestring, False),
+                   'logging': (Mapping, False)}
         self._check_keys(opts, keydict, stack, True)
 
     def _check_edge(self, opts, stack):
@@ -136,7 +137,7 @@ class ConfigValidation(object):
                    'ehlo_as': (basestring, False)}
         self._check_keys(opts, keydict, stack)
 
-    def _check_toplevel(self, stack):
+    def _check_toplevel(self, stack, program):
         keydict = {'process': (Mapping, True),
                    'edge': (Mapping, True),
                    'relay': (Mapping, True),
@@ -156,9 +157,13 @@ class ConfigValidation(object):
         for relay, opts in self.cfg.relay.iteritems():
             self._check_relay(opts, stack+['relay', relay])
 
+        if program not in self.cfg.process:
+            msg = "Missing required key '{0}'".format(program)
+            raise ConfigValidationError(msg, stack+['process'])
+
     @classmethod
-    def check(cls, cfg):
-        return cls(cfg)._check_toplevel(['root'])
+    def check(cls, cfg, program):
+        return cls(cfg)._check_toplevel(['root'], program)
 
 
 # vim:et:fdm=marker:sts=4:sw=4:ts=4
